# -*- coding: utf-8 -*-
"""
Created on Thu Mar  3 11:22:03 2022

@author: Wahb
"""
# In[0] - Importation de bibliothèques

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from sklearn.preprocessing import StandardScaler, MinMaxScaler
from sklearn.cluster import KMeans
import warnings
warnings.filterwarnings('ignore')

# In[1] - Prétraitement de données

## Importation de jeu de données
data = pd.read_excel(r'C:/Users/2016-1951/Desktop/5vs7.xlsx')   # importer le fichier de données excel
noms_params = list(data.columns)[:-1] + ["cluster"]
X = data.iloc[:,:-1]                                            # retirer la dernière colonne
#sc = StandardScaler()
#X = sc.fit_transform(X)     # fit : calculer et stocker la moyenne et l'écart-type de chaque variable; 
                             # transform : centrer et réduire les variables


# In[2] - Méthode du coude

## La méthode du coude est une heuristique utilisée pour déterminer le nombre de clusters dans un ensemble de données. 
## La méthode consiste à tracer l'inertie totale (somme des carrés des distances entre chaque individu et le centroïde du cluster)..
## ..en fonction du nombre de clusters et à choisir le coude de la courbe comme nombre de clusters à utiliser.

## Calcule d'inertie
Inertia = []
for nc in range(2,11):   # nombre de clusters à tester 
    km = KMeans(n_clusters=nc, init='random', n_init=10, max_iter=300, tol=1e-04, random_state=0)
    km.fit(X)
    Inertia.append(km.inertia_)    

# In[3] - Plot de la méthode du coude
   
plt.figure(figsize=(8,6))
plt.plot(range(2, 11), Inertia, marker='o', c='blue')
plt.xlabel('Nombre de clusters', size = 12)
plt.ylabel('Inertie totale', size = 12)
plt.title("Méthode du coude", size = 16)
plt.grid(linestyle=':')

plt.savefig("méthode_du_coude.pdf")
plt.show()

# In[4] - Modèle K-means optimal

## Dans cette partie, la méthode des K-moyennes est appliquée avec un nombre optimal de clusters.

## Entrainemet du modèle
nc = 5      # nombre de clusters à définir (si possible) à l'aide de la Méthode du coude
km = KMeans(n_clusters=nc, init='random', n_init=10, max_iter=300, tol=1e-04, random_state=0)
y_km = km.fit_predict(X)


# In[5] - 2D plot

''' Seulement pour les données 2D

## Plot 
plt.figure(figsize=(16,8))
#plt.xscale('log', base= 2); #plt.yscale('log', base= 2)
markers = ["o", "p", "v", "1", "P","x"]
colors = ["b", "g", "c", "m", "y", "sienna"]

    # plot des clusters
for i in range(nc):
    plt.scatter(X[y_km == i, 0], X[y_km == i, 1],s=50, c=colors[i], marker=markers[i])

    # plot des centroïdes
plt.scatter(km.cluster_centers_[:, 0], km.cluster_centers_[:, 1], s=75, marker='*',c='red')

plt.xlabel('Quantité', size = 12)
plt.ylabel('Nombre Commande', size = 12)
plt.title("Segmentation par K-moyennes", size = 16)
plt.grid(linestyle=':')
plt.show()
'''

# In[6] - Exporter les résultats vers fichier Excel

## Définir le cluster pour chaque individu
data_membership = np.hstack((X,km.labels_.reshape(-1,1)))
data_membership = pd.DataFrame(data=data_membership)

## Enregistrer les résultats dans un fichier excel
writer = pd.ExcelWriter('résultats_clustering.xlsx', engine='xlsxwriter') 
data_membership.to_excel(writer, index=False, header=noms_params, sheet_name='data_clustering')
writer.close()